#include "datebar.h"
#include <qdatetime.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qwmatrix.h>
#include <qfontmetrics.h>
#include "dateviewhandler.h"
#include <qtoolbutton.h>

const int histogramBarHeight = 30;
const int borderAboveHistogram = 6;
const int borderArroundWidget = 6;
const int buttonWidth = 22;

DateBar::DateBar( QWidget* parent, const char* name )
    :QWidget( parent, name ), _currentHandler( &_yearViewHandler ), _tp(YearView),_currentDate( QDateTime::currentDateTime() )
{
    setBackgroundMode( NoBackground );

    _rightArrow = new QToolButton( RightArrow, this );
    _rightArrow->setFixedSize( QSize( buttonWidth, histogramBarHeight ) );
    connect( _rightArrow, SIGNAL( clicked() ), this, SLOT( scrollRight() ) );

    _leftArrow = new QToolButton( LeftArrow, this );
    _leftArrow->setFixedSize( QSize( buttonWidth, histogramBarHeight ) );
    connect( _leftArrow, SIGNAL( clicked() ), this, SLOT( scrollLeft() ) );

    _zoomIn = new QToolButton( this );
    _zoomIn->setIconSet( QPixmap( "icons/viewmag+.png" ) );
    _zoomIn->setFixedSize( buttonWidth, buttonWidth );
    connect( _zoomIn, SIGNAL( clicked() ), this, SLOT( zoomIn() ) );
    connect( this, SIGNAL(canZoomIn(bool)), _zoomIn, SLOT( setEnabled( bool ) ) );

    _zoomOut = new QToolButton( this );
    _zoomOut->setIconSet( QPixmap( "icons/viewmag-.png" ) );
    _zoomOut->setFixedSize( buttonWidth, buttonWidth );
    connect( _zoomOut, SIGNAL( clicked() ), this, SLOT( zoomOut() ) );
    connect( this, SIGNAL(canZoomOut(bool)), _zoomOut, SLOT( setEnabled( bool ) ) );
}

QSize DateBar::sizeHint() const
{
    return QSize( 1024, 65 );
}

QSize DateBar::minimumSizeHint() const
{
    return QSize( 200, 65 );
}

void DateBar::paintEvent( QPaintEvent* /*event*/ )
{
    bitBlt( this, 0,0, &_buffer );
}

void DateBar::redraw()
{
    if ( _buffer.isNull() )
        return;

    QPainter p( &_buffer );
    QColorGroup grp = palette().active();

    // Fill with background pixels
    p.save();
    p.setPen( grp.dark() );
    p.setBrush( grp.background() );
    p.drawRect( rect() );

    // Draw the area with histograms
    QRect barArea = barAreaGeometry();

    p.setBrush( grp.base() );
    p.drawRect( barArea );
    p.restore();

    _currentHandler->init( _currentHandler->date( -_currentUnit, _currentDate ) );

    QRect tickMarkRect;
    tickMarkRect.setTopLeft( barArea.bottomLeft() );
    tickMarkRect.setWidth( barArea.width() );
    tickMarkRect.setHeight( 12 );
    drawTickMarks( p, tickMarkRect );
    drawHistograms( p, barArea );
    drawFocusRectagle( p, barArea );
    update();
}

void DateBar::resizeEvent( QResizeEvent* event )
{
    _leftArrow->move( borderArroundWidget, borderAboveHistogram );
    _rightArrow->move( event->size().width() - _rightArrow->width() - borderArroundWidget, borderAboveHistogram );

    int x = _leftArrow->pos().y();
    int y = _rightArrow->geometry().bottom() + 3;
    _zoomOut->move( x, y );

    x = _rightArrow->pos().x();
    _zoomIn->move(x, y );

    _buffer.resize( event->size() );
    _currentUnit = numberOfUnits()/2;
    redraw();
}

void DateBar::drawTickMarks( QPainter& p, const QRect& rect )
{
    p.save();
    p.setPen( QPen( palette().active().text(), 1 ) );

    QFont f( font() );
    QFontMetrics fm(f);
    int fontHeight = fm.height();
    int unit = 0;
    QRect clip = rect;
    clip.setHeight( rect.height() + 2 + fontHeight );
    clip.setLeft( clip.left() + 2 );
    clip.setRight( clip.right() -2 );
    p.setClipRect( clip );

    for ( int x = rect.x(); x < rect.right(); x+=tickWidth, unit += 1 ) {
        int h = rect.height();
        if ( _currentHandler->isMajorUnit( unit ) ) {
            QString text = _currentHandler->text( unit );
            int w = fm.width( text );
            p.setFont( f );
            p.drawText( x - w/2, rect.bottom()+2, w, fontHeight, Qt::SingleLine, text );
        }
        else if ( _currentHandler->isMidUnit( unit ) )
            h = (int) ( 2.0/3*rect.height());
        else
            h = (int) ( 1.0/3*rect.height());

        p.drawLine( x, rect.top(), x, rect.top() + h );
    }

    p.restore();
}

void DateBar::setViewType( ViewType tp )
{
    switch ( tp ) {
    case DecadeView: _currentHandler = &_decadeViewHandler; break;
    case YearView: _currentHandler = &_yearViewHandler; break;
    case MonthView: _currentHandler = &_monthViewHandler; break;
    case WeekView: _currentHandler = &_weekViewHandler; break;
    case DayView: _currentHandler = &_dayViewHandler; break;
    case HourView: _currentHandler = &_hourViewHandler; break;
    }
    redraw();
    _tp = tp;
}

void DateBar::setDate( const QDateTime& date )
{
    _currentDate = date;
    redraw();
}

void DateBar::setData( const ImageDateRangeCollection& dates )
{
    _dates = dates;
}

void DateBar::drawHistograms( QPainter& p, const QRect& rect )
{
    p.save();
    p.setClipping( true );
    p.setClipRect( rect );
    p.setPen( NoPen );

    int unit = 0;
    int max = 0;
    for ( int x = rect.x(); x < rect.right(); x+=tickWidth, unit += 1 ) {
        ImageCount count = _dates.count( _currentHandler->date(unit), _currentHandler->date(unit+1).addSecs(-1) );
        max = QMAX( max, count._exact + count._rangeMatch  );
    }

    unit = 0;
    for ( int x = rect.x(); x < rect.right(); x+=tickWidth, unit += 1 ) {
        ImageCount count = _dates.count( _currentHandler->date(unit), _currentHandler->date(unit+1).addSecs(-1) );
        int exact = (int) ((double) (rect.height()-2) * count._exact / max );
        int range = (int) ((double) (rect.height()-2) * count._rangeMatch / max );

        p.setBrush( blue.light() );
        p.drawRect( x+1, rect.bottom()-range, tickWidth-2, range );
        p.setBrush( blue );
        p.drawRect( x+1, rect.bottom()-range-exact, tickWidth-2, exact );

    }

    p.restore();
}

void DateBar::scrollLeft()
{
    scroll( -1 );
}

void DateBar::scrollRight()
{
    scroll( 1 );
}

void DateBar::scroll( int units )
{
    _currentDate = _currentHandler->date( units, _currentDate );
    redraw();
}

void DateBar::drawFocusRectagle( QPainter& p, const QRect& rect)
{
    p.save();
    int x = rect.left() + _currentUnit*tickWidth;
    QRect inner( QPoint(x-1, borderAboveHistogram),
                 QPoint( x + tickWidth, borderAboveHistogram + histogramBarHeight - 1 ) );

    p.setPen( QPen( palette().active().dark(), 1 ) );

    // Inner rect
    p.drawRect( inner );
    QRect outer = inner;
    outer.addCoords( -2, -2, 2, 2 );

    // Outer rect
    QRegion region = outer;
    region -= inner;
    p.setClipping( true );
    p.setClipRegion( region );

    p.setBrush( red );
    p.setPen( red );
    p.drawRect( outer );

    // Shadow below
    QRect shadow = outer;
    shadow.addCoords( -1,-1, 1, 1 );
    region = shadow;
    region -= outer;
    p.setPen( palette().active().shadow() );
    p.setClipRegion( region );
    p.drawRect( shadow );

    // Light above
    QRect hide = shadow;
    hide.moveBy( 1, 1 );
    region = shadow;
    region -= hide;
    p.setPen( palette().active().light() );
    p.setClipRegion( region );
    p.drawRect( shadow );

    p.restore();
}

void DateBar::zoomIn()
{
    if ( _tp == HourView )
        return;
    zoom(+1);
}

void DateBar::zoomOut()
{
    if ( _tp == DecadeView )
        return;
    zoom(-1);
}

void DateBar::zoom( int factor )
{
    ViewType tp = (ViewType) (_tp+factor);
    setViewType( tp );
    emit canZoomIn( tp != HourView );
    emit canZoomOut( tp != DecadeView );
}

void DateBar::mousePressEvent( QMouseEvent* event )
{
    _movingBar = event->y() > barAreaGeometry().bottom();
    if ( _movingBar )
        _movementOffset = _currentUnit * tickWidth - ( event->pos().x() - barAreaGeometry().left() );
    else {
        _currentUnit = ( event->pos().x()  - barAreaGeometry().left() )/tickWidth;
        _currentDate = _currentHandler->date( _currentUnit );
        _movementOffset = 0;
    }
    redraw();
}

void DateBar::mouseMoveEvent( QMouseEvent* event )
{
    _currentUnit = ( event->pos().x() + _movementOffset - barAreaGeometry().left() )/tickWidth;
    if ( _currentUnit < 0 )
        _currentUnit = 0;

    if ( _currentUnit > numberOfUnits() )
        _currentUnit = numberOfUnits();

    if ( !_movingBar )
        _currentDate = _currentHandler->date( _currentUnit );
    redraw();
}

QRect DateBar::barAreaGeometry() const
{
    QRect barArea;
    barArea.setTopLeft( QPoint( borderArroundWidget + buttonWidth + 2, borderAboveHistogram ) );
    barArea.setRight( width() - borderArroundWidget - buttonWidth - 3 ); // 2 pixels between button and bar + 1 pixel as the pen is one pixel
    barArea.setHeight( histogramBarHeight );
    return barArea;
}

int DateBar::numberOfUnits() const
{
    return barAreaGeometry().width() / tickWidth -1 ;
}

